==> crate-anon-0.18.97.tar.gz: crate-anon-0.18.97/crate_anon/common/stringfunc.py <==
#!/usr/bin/env python

"""
crate_anon/common/stringfunc.py

===============================================================================

    Copyright (C) 2015-2020 Rudolf Cardinal (rudolf@pobox.com).

    This file is part of CRATE.

    CRATE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    CRATE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with CRATE. If not, see <http://www.gnu.org/licenses/>.

===============================================================================

**Simple string functions.**

"""

import fnmatch
from functools import lru_cache
import sys
from typing import Any, Pattern, TextIO

import regex


# =============================================================================
# Simple string manipulation
# =============================================================================

def get_digit_string_from_vaguely_numeric_string(s: str) -> str:
    """
    Strips non-digit characters from a string.

    For example, converts ``"(01223) 123456"`` to ``"01223123456"``.
    """
    return "".join([d for d in s if d.isdigit()])


def reduce_to_alphanumeric(s: str) -> str:
    """
    Strips non-alphanumeric characters from a string.

    For example, converts ``"PE12 3AB"`` to ``"PE12 3AB"``.
    """
    return "".join([d for d in s if d.isalnum()])


def remove_whitespace(s: str) -> str:
    """
    Removes whitespace from a string.
    """
    return ''.join(s.split())


# =============================================================================
# Specification matching
# =============================================================================

@lru_cache(maxsize=None)
def get_spec_match_regex(spec: str) -> Pattern:
    """
    Returns a compiled, case-insensitive regular expression representing a
    shell-style pattern (using ``*``, ``?`` and similar wildcards; see
    https://docs.python.org/3.5/library/fnmatch.html).

    Args:
        spec: the pattern to pass to ``fnmatch``, e.g. ``"patient_addr*"``.

    Returns:
        the compiled regular expression
    """
    return regex.compile(fnmatch.translate(spec), regex.IGNORECASE)


# =============================================================================
# Printing/encoding
# =============================================================================

def uprint(*objects: Any,
           sep: str = ' ',
           end: str = '\n',
           file: TextIO = sys.stdout) -> None:
    """
    Prints strings to outputs that support UTF-8 encoding, but also to those
    that do not (e.g. Windows stdout, sometimes).

    Args:
        *objects: things to print
        sep: separator between those objects
        end: print this at the end
        file: file-like object to print to

    See
    http://stackoverflow.com/questions/14630288/unicodeencodeerror-charmap-codec-cant-encode-character-maps-to-undefined

    Examples:

    - Linux, Python 3.6.8 console: ``sys.stdout.encoding == "UTF-8"``
    - Windows, Python 3.7.4 console: ``sys.stdout.encoding == "utf-8"``
    - Windows, Python 3.7.4, from script: ``sys.stdout.encoding == "cp1252"``
    """  # noqa
    enc = file.encoding.lower()
    if enc == 'utf-8':
        print(*objects, sep=sep, end=end, file=file)
    else:
        def f(obj: Any) -> str:
            return str(obj).encode(enc, errors='backslashreplace').decode(enc)
        # https://docs.python.org/3.5/library/codecs.html#codec-base-classes
        print(*map(f, objects), sep=sep, end=end, file=file)


# =============================================================================
# String tests
# =============================================================================

def does_text_contain_word_chars(text: str) -> bool:
    """
    Is a string worth treating as interesting text -- does it contain "word"
    characters?
    """
    # Slower (as per FS's tests):
    #   regex_any_word_char = regex.compile(r'[\w\W]*[a-zA-Z0-9_][\w\W]*')
    #   return bool(text and regex_any_word_char.match(text))
    # Faster:
    return bool(text and any(33 <= ord(c) <= 126 for c in text))

==> crate-anon-0.19.2.tar.gz: crate-anon-0.19.2/crate_anon/common/stringfunc.py <==
#!/usr/bin/env python

"""
crate_anon/common/stringfunc.py

===============================================================================

    Copyright (C) 2015-2021 Rudolf Cardinal (rudolf@pobox.com).

    This file is part of CRATE.

    CRATE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    CRATE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with CRATE. If not, see <http://www.gnu.org/licenses/>.

===============================================================================

**Simple string functions.**

"""

import fnmatch
from functools import lru_cache
import sys
from typing import Any, Pattern, TextIO

import regex


# =============================================================================
# Simple string manipulation
# =============================================================================

def get_digit_string_from_vaguely_numeric_string(s: str) -> str:
    """
    Strips non-digit characters from a string.

    For example, converts ``"(01223) 123456"`` to ``"01223123456"``.
    """
    return "".join([d for d in s if d.isdigit()])


def reduce_to_alphanumeric(s: str) -> str:
    """
    Strips non-alphanumeric characters from a string.

    For example, converts ``"PE12 3AB"`` to ``"PE12 3AB"``.
    """
    return "".join([d for d in s if d.isalnum()])


def remove_whitespace(s: str) -> str:
    """
    Removes whitespace from a string.
    """
    return "".join(s.split())


# =============================================================================
# Specification matching
# =============================================================================

@lru_cache(maxsize=None)
def get_spec_match_regex(spec: str) -> Pattern:
    """
    Returns a compiled, case-insensitive regular expression representing a
    shell-style pattern (using ``*``, ``?`` and similar wildcards; see
    https://docs.python.org/3.5/library/fnmatch.html).

    Args:
        spec: the pattern to pass to ``fnmatch``, e.g. ``"patient_addr*"``.

    Returns:
        the compiled regular expression
    """
    return regex.compile(fnmatch.translate(spec), regex.IGNORECASE)


# =============================================================================
# Printing/encoding
# =============================================================================

def uprint(*objects: Any,
           sep: str = ' ',
           end: str = '\n',
           file: TextIO = sys.stdout) -> None:
    """
    Prints strings to outputs that support UTF-8 encoding, but also to those
    that do not (e.g. Windows stdout, sometimes).

    Args:
        *objects: things to print
        sep: separator between those objects
        end: print this at the end
        file: file-like object to print to

    See
    http://stackoverflow.com/questions/14630288/unicodeencodeerror-charmap-codec-cant-encode-character-maps-to-undefined

    Examples:

    - Linux, Python 3.6.8 console: ``sys.stdout.encoding == "UTF-8"``
    - Windows, Python 3.7.4 console: ``sys.stdout.encoding == "utf-8"``
    - Windows, Python 3.7.4, from script: ``sys.stdout.encoding == "cp1252"``
    """  # noqa
    enc = file.encoding.lower()
    if enc == 'utf-8':
        print(*objects, sep=sep, end=end, file=file)
    else:
        def f(obj: Any) -> str:
            return str(obj).encode(enc, errors='backslashreplace').decode(enc)
        # https://docs.python.org/3.5/library/codecs.html#codec-base-classes
        print(*map(f, objects), sep=sep, end=end, file=file)


# =============================================================================
# String tests
# =============================================================================

def does_text_contain_word_chars(text: str) -> bool:
    """
    Is a string worth treating as interesting text -- does it contain "word"
    characters?
    """
    # Slower (as per FS's tests):
    #   regex_any_word_char = regex.compile(r'[\w\W]*[a-zA-Z0-9_][\w\W]*')
    #   return bool(text and regex_any_word_char.match(text))
    # Faster:
    return bool(text and any(33 <= ord(c) <= 126 for c in text))
